# MSE protocol #

## version: 1.0 ##

This protocol is used by [media server engine (MSE)](https://bitbucket.org/telesessionteam/telesession-media-core/overview) and clients for communication. Protocol is based on [JSON RPC 2.0](http://www.jsonrpc.org/specification).

## Example usage ##

* Install package using NPM:

```bash
npm i --save mse-proto
```

```javascript
const WebSocket = require('ws')
const proto = require('mse-proto')
const RequestFactory = proto.RequestFactory
const ResponseAdapter = proto.ResponseAdapter

const wss = new WebSocket(`wss://mse:443/`)

wss.on('open', (ws) => {
  wss.send(RequestFactory.ping())
})
wss.on('message', (response) => {
  if (ResponseAdaptor.ping(response)) {
    console.log(response)
  }
})
```

----

## Messages ##

* [ping](#ping)
* [authentication](#authentication)
* [whoami](#whoami)
* [iceCandidate](#iceCandidate)
* [sdpOffer](#sdpOffer)
* [disconnect](#disconnect)

## Errors ##

* [Invalid json object](#invalid-json-object)
* [Method not found](#Method-not-found)
* [Invalid method parameter](#Invalid-method-parameter)
* [Invalid json rpc 2.0 object](#Invalid-json-rpc-2.0-object)
* [Authentication failure](#Authentication-failure)
* [Unauthorized access](#Unauthorized-access)
* [Invalid call order](#Invalid-call-order)

----

## ping ##

### ping request (client) ###

```json
{
  "id": 1,
  "method": "ping",
  "jsonrpc": "2.0"
}
```

### ping success response (server) ###

```json
{
  "id": 1,
  "result": "pong",
  "jsonrpc": "2.0"
}
```

## authentication ##

### authentication request (client) ###

* A JWT token is used for authentication. Subsequent calls do NOT need the token.

```json
{
  "id": 2,
  "method": "auth",
  "jsonrpc": "2.0",
  "params": {
    "token": "appropriate jwt token",
    "version": "1.0"
  }
}
```

### authentication success response (server) ###

```json
{
  "id": 2,
  "jsonrpc": "2.0",
  "result": "authenticated"
}
```

## whoami ##

### whoami request (client) ###

* Available in **development** environment only.
* This method is available only to **authorized** clients.

```json
{
  "id": 3,
  "method": "whoami",
  "jsonrpc": "2.0"
}
```

### whoami success response (server) ###

```json
{
  "id": 3,
  "jsonrpc": "2.0",
  "result": {
    "domain": "domain",
    "agent": "web or mobile",
    "role": "user - admin or super",
    "userId": "random string",
    "roomId": "random string",
    "ipAddress": "client ip address",
    "tokenIssued": "date the token is issued",
    "validFrom": "token is valid from date",
    "validTo": "token is valid to date"
  }
}
```

## iceCandidate ##

### iceCandidate request (client) ###

* This method is available only to **authorized** clients.
* This method is a notification. Server does NOT respond to it.

```json
{
  "id": null,
  "method": "iceCandidate",
  "jsonrpc": "2.0",
  "params": {
    "candidate": "ice candidate value"
  }
}
```

## sdpOffer ##

### sdpOffer request (client) ###

* This method is available only to **authorized** clients.
* This method is a notification. Server does NOT respond to it.

```json
{
  "id": null,
  "method": "sdpOffer",
  "jsonrpc": "2.0",
  "params": {
    "offer": "sdp offer value"
  }
}
```

## disconnect ##

### disconnect request (client) ###

* This method is available only to **authorized** clients.
* This method is a notification. Server does NOT respond to it.

```json
{
  "id": null,
  "method": "disconnect",
  "jsonrpc": "2.0"
}
```

### Chat message (client & server) ###

* NOT IMPLEMENTED
* This method is available only to **authorized** clients.
* This method is a notification. Server does NOT respond to it.

```json
{
  "id": null,
  "method": "chat",
  "jsonrpc": "2.0",
  "params": {
    "to": [],
    "message": "message",
    "attachment": "object"
  }
}
```

----

## Errors ##

### Invalid json object ###

```json
{
  "id": null
  "jsonrpc": "2.0",
  "error": {
    "code": -32700,
    "message": "Parse error"
  }
}
```

### Invalid json rpc 2.0 object ###

```json
{
  "id": null
  "jsonrpc": "2.0",
  "error": {
    "code": -1001,
    "message": "Invalid json rpc 2.0 object"
  }
}
```

### Method not found ###

```json
{
  "id": "related request id",
  "jsonrpc": "2.0",
  "error": {
    "code": -32601,
    "message": "Method not found"
  }
}
```

### Invalid method parameter ###

```json
{
  "id": "related request id",
  "jsonrpc": "2.0",
  "error": {
    "code": -32602,
    "message": "Invalid params"
  }
}
```

### Authentication failure ###

```json
{
  "id": "related request id",
  "jsonrpc": "2.0",
  "error": {
    "code": -4001,
    "message": "Authorization Failed"
  }
}
```

### Unauthorized access ###

* Accessing a protected method before [authorization](#authorization).

* Available methods before authentication:

  * [ping](#ping)
  * [auth](#authentication)

```json
{
  "id": "related request id",
  "jsonrpc": "2.0",
  "error": {
    "code": -5001,
    "message": "Unauthorized Access"
  }
}
```

### Invalid call order ###

* Calling a method before calling required methods before it.

```json
{
  "id": 3,
  "jsonrpc": "2.0",
  "error": {
    "code": -6001,
    "message": "Invalid Call Order",
    "data": {
      "required": "Required Method"
    }
  }
}
```
