/* http://www.jsonrpc.org/specification */

class ResponseFactory {
  static invalidJson (msg) {
    let jsonRpcMessage = new JsonRpcMessage(null)
    let error = new JsonRpcError(-32700, 'Parse error', {originalMessage: msg})
    jsonRpcMessage.error = error
    return JSON.stringify(jsonRpcMessage)
  }

  static invalidJsonRpc (msg) {
    let jsonRpcMessage = new JsonRpcMessage(null)
    let error = new JsonRpcError(-1001, 'Invalid json rpc 2.0 object', {originalMessage: msg})
    jsonRpcMessage.error = error
    return JSON.stringify(jsonRpcMessage)
  }

  static invalidRequest (msg) {
    let jsonRpcMessage = new JsonRpcMessage(null)
    let error = new JsonRpcError(-32600, 'Invalid Request', {originalMessage: msg})
    jsonRpcMessage.error = error
    return JSON.stringify(jsonRpcMessage)
  }

  static ping (msg) {
    let obj = this.safeParse(msg)
    let jsonRpcMessage = new JsonRpcMessage(obj.id)
    jsonRpcMessage.result = 'pong'
    return JSON.stringify(jsonRpcMessage)
  }

  static whoAmI (msg, user) {
    let obj = this.safeParse(msg)
    let jsonRpcMessage = new JsonRpcMessage(obj.id)
    jsonRpcMessage.result = {
      userId: user.id,
      roomId: user.roomId,
      ip: user.ip,
      domain: user.domain,
      agent: user.agent,
      role: user.role,
      options: user.options,
      tokenIssued: new Date(user.tokenIssued).toUTCString(),
      validFrom: new Date(user.validFrom).toUTCString(),
      validTo: new Date(user.validTo).toUTCString()
    }
    return JSON.stringify(jsonRpcMessage)
  }

  static auth (msg) {
    let obj = JSON.parse(msg)
    if (obj.params.token === 'pass') {
      let jsonRpcMessage = new JsonRpcMessage(obj.id)
      jsonRpcMessage.result = 'authenticated'
      return JSON.stringify(jsonRpcMessage)
    } else {
      return this.unAuthorized(msg)
    }
  }

  static unAuthorized (msg) {
    let obj = JSON.parse(msg)
    let error = new JsonRpcError(-4001, 'Authorization Failed', {originalMessage: msg})
    let jsonRpcMessage = new JsonRpcMessage(obj.id)
    jsonRpcMessage.error = error
    return JSON.stringify(jsonRpcMessage)
  }

  static chatMessage (sender, message, type = 'public') {
    let jsonRpcMessage = new JsonRpcMessage('chat')
    jsonRpcMessage.params = {
      type: type ? `${type}` : 'private',
      sender: sender,
      message: message
    }
    return JSON.stringify(jsonRpcMessage)
  }

  static addIceCandidate (candidate) {
    let jsonRpcMessage = new JsonRpcMessage('invoke')
    jsonRpcMessage.params = {
      operation: 'addIceCandidate',
      object: 'peer',
      operationParams: {
        'candidate': candidate
      }
    }
    return JSON.stringify(jsonRpcMessage)
  }

  static protectedMethod (msg) {
    let obj = JSON.parse(msg)
    let error = new JsonRpcError(-5001, 'Unauthorized Access', {originalMessage: msg})
    let jsonRpcMessage = new JsonRpcMessage(obj.id)
    jsonRpcMessage.error = error
    return JSON.stringify(jsonRpcMessage)
  }

  static invalidCallOrder (msg) {
    let obj = JSON.parse(msg)
    let error = new JsonRpcError(-6001, 'Invalid Call Order', {originalMessage: msg})
    let jsonRpcMessage = new JsonRpcMessage(obj.id)
    jsonRpcMessage.error = error
    return JSON.stringify(jsonRpcMessage)
  }

  static invalidMethodParameter (msg) {
    let obj = JSON.parse(msg)
    let error = new JsonRpcError(-32602, 'Invalid Method Parameter', {originalMessage: msg})
    let jsonRpcMessage = new JsonRpcMessage(obj.id)
    jsonRpcMessage.error = error
    return JSON.stringify(jsonRpcMessage)
  }

  static methodNotFound (msg) {
    let obj = JSON.parse(msg)
    let error = new JsonRpcError(-32601, 'Method not found', {originalMessage: msg})
    let jsonRpcMessage = new JsonRpcMessage(obj.id)
    jsonRpcMessage.error = error
    return JSON.stringify(jsonRpcMessage)
  }

  static disconnect (msg, user) {
    let jsonRpcMessage = new JsonRpcMessage(msg.id)
    jsonRpcMessage.result = 'disconnect'
    // total online time
    return JSON.stringify(jsonRpcMessage)
  }

  static createSomething () {
    let jsonRpcMessage = new JsonRpcMessage('create')
    jsonRpcMessage.params = {
      type: 'Something',
      constructorParams: { }
    }
    return JSON.stringify(jsonRpcMessage)
  }
  static safeParse (str) {
    try {
      const obj = JSON.parse(str)
      if (obj && typeof obj === 'object' && obj !== null) {
        return obj
      }
    } catch (err) {}
    return null
  }
}

/* Helper classes */

class JsonRpcNotification {
  constructor () {
    this.jsonrpc = '2.0'
  }
}

class JsonRpcMessage extends JsonRpcNotification {
  constructor (id = null) {
    super()
    this.id = id
  }
}

/*
-32700 Parse error      : Invalid JSON was received by the server. An error occurred on the server while parsing the JSON text.
-32600 Invalid Request  : The JSON sent is not a valid Request object.
-32601 Method not found : The method does not exist / is not available.
-32602 Invalid params   : Invalid method parameter(s).
-32603 Internal error   : Internal JSON-RPC error.
-32000 to -32099        : Server error. Reserved for implementation-defined server-errors.
*/
class JsonRpcError {
  constructor (code = null, message = 'error', data = null) {
    this.code = code
    this.message = message
    this.data = data
  }
}

module.exports = ResponseFactory
